/**
 * @class
 * @abstract
 * @classdesc Each command should extend this superclass.
 * Normaly you'd expect in a command pattern that subclasses must override the
 * 'execute' method with their specific implementations. You could of course
 * inject the context to the instructor as a parameter, and do the error/type checking in
 * the constructor as well. The reason that the execute method already has a final implementation, is because
 * each command should return a result object. You can't force that via the constructor, or you'll have to run the
 * command immediately after instantiation. With the current implementation, I can first create a new command, and
 * execute it later on.
 * @author ruckola
 * @property {CommandResult} this.result
 * @property {CommandContext} this.ctxt
 */
class Command {

    /**
     * @constructor
     */
    constructor() {
        if (this.constructor === Command) {
            throw new Error("The Command class is abstract. You can't instantiate it!");
        }
        var CommandResult = require('../core/commandResult');
        this.result = new CommandResult();
        this.context = {};
    }

    /**
     * @method execute
     * @param {CommandContext} context - The context object is used to inject the data
     * that is required to successfully execute the command.
     * @description The execute method must be used to run the command. It is responsible
     * for invoking the task() method and returning a value of the right type.
     * @returns {Promise} - The value of the promise should be this.result in most cases. 
     */
    execute(context) {

    }

    /**
     * @method
     * @description
     * If the execution fails before the command task() has returned a promise,
     * we have to return a rejected promise manually to the client. Since there
     * is no data to return yet, we pass an error object as the value.
     * @param {Error} err - An error object that will be set as the value for the rejected promise.
     * @returns {Promise} - Allways rejects with an error object as its value.   
     */
    abort(err) {
        return new Promise(function (resolve, reject) {
            reject(err);
        });
    }
}

module.exports = Command;