/**
 * @class
 * @classdesc
 * The CommandContext is used to provide a command with additional information
 * that it requires in order to fullfill its task successfully.
 * You can use a server request as commandContext or create it manually,
 * by adding your own parameters via the addParameter method.
 * @author ruckola
 * @property {object} parameters - Contains the key-value pairs that are needed to run a command.
 * @property {string} parameters.action - The name of the command to be invoked, sets {@link CommandContext#action}.
 * @property {number} parameters.returnMode - 0 (default) for normal object | 1 for promise {@link CommandContext#returnMode}
 * @property {Object} parameters.request - Server request (optional) {@link CommandContext#request}
 */
class CommandContext {

    /**
     * @constructor
     * @param {object} request - A server request object with an 'action' member.
     * for example {action:'aCommandName'}
     * @returns parameters
     * @description The CommandContext is an object that is passed as a parameter
     * to each new command at instantiation. You can add new parameters to your own needs.
     * The default context parameters are:
     * action:'default' (the default command)
     * request: server request
     * (If a server request is available it will be added to the optional
     * request member.)
     * userConfigFile: The path to the file with user defined macro template definitions.
     * userCommandsRootDir: The user project directory with user defined commands.
     * userHelperDir: User defined modules that are used to support user commands.
     * userConfiguration: The content of the userConfigFile. 
     */
    constructor(request = undefined) {

        /**
         * @name CommandContext#parameters
         * @type Object
         * @default Null
         */
        this.parameters = {};

        /**
         * @name CommandContext#action
         * @type string
         * @default "default"
         */
        this.parameters.action = "macro";

        /**
         * @name CommandContext#macroID
         * @type Number
         * @default False
         */
        this.parameters.macroID = false;

        if (typeof request !== 'undefined' &&
            Object.prototype.toString.call(request) === '[object Object]') {
            
            /**
             * @name CommandContext#request
             * @type Object
             * @default false
             */
            this.parameters.request = request;
        } else {
            this.parameters.request = false;
        }

        /**
         * @name CommandContext#userConfigFile
         * @type string
         * @default false
         */
        this.parameters.userConfigFile = false;

        /**
         * @name CommandContext#userCommandsRootDir
         * @type string
         * @default false
         */
        this.parameters.userCommandsRootDir = false;

        /**
         * @name CommandContext#userHelperDir
         * @type string
         * @default false
         */
        this.parameters.userHelperDir = false;

        /**
         * @name CommandContext#userConfiguration
         * @type string
         * @default false
         */
        this.parameters.userConfiguration = false;

    }

    /**
     * @method
     * @param {string} key - Name of the new key that will be added to the context parameters.
     * @param {any} value - Any value for the new key;
     * @description Add a new member (key-value pair) to the context parameters.
     */
    addParameter(key, value) {
        this.parameters[key] = value;
    }

    /**
     * @method
     * @param {string} key - Name of the member to return.
     * @description Add a new member (key-value pair) to the context parameters.
     * @return {value} The value of member[key].
     */
    get(key) {
        return this.parameters[key];
    }
}

module.exports = CommandContext;